@@ -902,13 +902,19 @@ cluster_iss <- function(tm,
   mc <- match.call()
   vcov_type <- match.arg(mc$vcov_type, c("MB", "HC", "CR", "DB")) # this sets the default to model-based
   if (vcov_type != "DB") {
-    # get gradient for ctrl means model
+    # get gradient for ctrl means model; only keep full-rank columns
     cm <- x@ctrl_means_model
-    cm_mm <- stats::model.matrix(formula(cm), stats::model.frame(cm, na.action = na.pass))
+    K <- cm$rank
+    piv <- cm$qr$pivot[seq_len(K)]
+    cm_mm <- stats::model.matrix(formula(cm),
+                                 stats::model.frame(cm, na.action = na.pass))
     cm_grad <- matrix(0, nrow = nrow(cm_mm), ncol = ncol(cm_mm),
-                      dimnames = list(NULL, paste(formula(x)[[2]], colnames(cm_mm), sep = ":")))
+                      dimnames = list(NULL, paste(formula(x)[[2]],
+                                                  colnames(cm_mm),
+                                                  sep = ":")))
     colnames(cm_mm) <- paste("cov_adj", colnames(cm_mm), sep = ":")
-    if (inherits(cm, "mlm")) cm_grad <- cbind(cm_grad, cm_mm)
+    if (inherits(cm, "mlm")) cm_grad <- cbind(cm_grad[,piv,drop=FALSE],
+                                              cm_mm[,piv,drop=FALSE])
     cm_wts <- replace(cm_wts <- stats::weights(cm), is.na(cm_wts), 0)
     wZ <- cbind(wZ, -cm_grad * cm_wts)
   }
@@ -953,11 +959,10 @@ cluster_iss <- function(tm,
 
 #' @title (Internal) Design-based variance estimates with HC0 adjustment
 #' @param x a fitted \code{teeMod} model
-#' @details The design-based variance estimates can be calculated for
-#' \code{teeMod} models satisfying the following requirements:
+#' @details Design-based variance estimation is implemented for
+#' \code{teeMod} models satisfying requirements including:
 #' - The model uses \code{rct_spec} as \code{StudySpecification}
 #' - The model only estimates a main treatment effect
-#' - Inverse probability weighting is incorporated
 #'
 #' @keywords internal
 #' @rdname var_estimators
@@ -975,10 +980,12 @@ cluster_iss <- function(tm,
     if (!all(os@keys$in_Q)){
       stop(paste("Design-based standard errors are not supported for teeMod",
                  "models with external sample for covariance adjustment"))
+      # Our theory does not support this calculation
     }
 
   if (x@StudySpecification@type != "RCT"){
-    stop("Design-based standard errors can only be computed for RCT specifications")
+    stop("Design-based standard errors implemented only for RCT specifications")
+    # The theory for design-based SE has not been extended to other specifications
   }
 
   args <- list(...)
@@ -1000,10 +1007,11 @@ cluster_iss <- function(tm,
   else {
     if (length(x@moderator) > 0){
       stop(paste("Design-based standard errors are not supported for teeMod",
-                 "models with moderators"))
+                 "models with moderators and without absorbed intercepts"))
+      # The theory for such models has not been derived
     }
     
-    # if model weights does not incorporate IPW, throw a warning
+    # If model weights does not incorporate IPW, throw a warning
     if (!(inherits(x@lmitt_call$weights, "call") &
           sum(grepl("ate", x@lmitt_call$weights)) > 0)){
       warning(paste("When calculating design-based standard errors,",
@@ -1045,14 +1053,17 @@ cluster_iss <- function(tm,
 #' @keywords internal
 .get_DB_covadj_se <- function(x, ...){
   if (x@absorbed_intercepts) {
-      stop(paste("Design-based standard errors are not supported for\n",
-                 "tee models with absorbed intercepts"))
+    stop(paste("x must be a tee model with covariance adjustment and\n",
+               "without absorbed intercepts"))
+    # For tee models with both covariance adjustment and absorbed intercepts,
+    # use vcov_tee or .vcov_DB0 for variance estimation
   }
   specification_obj <- x@StudySpecification
   name_trt <- var_names(specification_obj, "t")
   if (name_trt %in% all.vars(stats::formula(x$model$`(offset)`@fitted_covariance_model))) {
     stop(paste("Design-based standard errors are not supported for\n",
                "tee models with treatment in prior covariance adjustment"))
+    # Our theory does not support this calculation
   }
 
   bread <- .get_DB_covadj_bread(x, ...)
